'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var apolloClient = require('apollo-client');
var apolloLinkHttp = require('apollo-link-http');
var apolloCacheInmemory = require('apollo-cache-inmemory');
var fetch = _interopDefault(require('isomorphic-fetch'));
var getConfig = _interopDefault(require('next/config'));
var Head = _interopDefault(require('next/head'));
var reactApollo = require('react-apollo');
var React = _interopDefault(require('react'));

const { publicRuntimeConfig } = getConfig();
let artemisClient = null;
const isBrowser = typeof window !== undefined;
if (!isBrowser) {
    global.fetch = fetch;
}
function createArtemis(initialState) {
    return new apolloClient.ApolloClient({
        connectToDevTools: isBrowser,
        ssrMode: !isBrowser,
        link: new apolloLinkHttp.HttpLink({
            uri: publicRuntimeConfig.graphQLEndpoint || "http://localhost:5000/graphql",
            credentials: "same-origin",
        }),
        cache: new apolloCacheInmemory.InMemoryCache().restore(initialState || {}),
    });
}
function initArtemis(initialState) {
    // create a new client for each server-side request
    // avoids shared data connections
    if (!isBrowser) {
        return createArtemis(initialState);
    }
    if (!artemisClient) {
        artemisClient = createArtemis(initialState);
    }
    return artemisClient;
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

const isBrowser$1 = typeof window !== undefined;
var WithArtemis = (App) => {
    var _a;
    return _a = class Artemis extends React.Component {
            constructor(props) {
                super(props);
                this.artemisClient = props.artemisClient || initArtemis(props.artemisState);
            }
            static getInitialProps(ctx) {
                return __awaiter(this, void 0, void 0, function* () {
                    const { Component, router } = ctx;
                    let appProps = {};
                    if (App.getInitialProps) {
                        appProps = yield App.getInitialProps(ctx);
                    }
                    let artemisState = {};
                    const artemis = initArtemis();
                    try {
                        yield reactApollo.getDataFromTree(React.createElement(App, Object.assign({}, appProps, { Component: Component, router: router, apolloState: artemisState, apolloClient: artemis })));
                    }
                    catch (error) {
                        // tslint:disable-next-line:no-console
                        console.error("Error while running `getDataFromTree`", error);
                    }
                    if (!isBrowser$1) {
                        // getDataFromTree does not call componentWillUnmount
                        // we need to clear the <Head> manually
                        Head.rewind();
                    }
                    artemisState = artemis.cache.extract();
                    return Object.assign({}, appProps, { artemisState });
                });
            }
            render() {
                return React.createElement(App, Object.assign({}, this.props, { artemisClient: this.artemisClient }));
            }
        },
        _a.displayName = "withArtemis(App)",
        _a;
};

exports.initArtemis = initArtemis;
exports.default = WithArtemis;
